#ifndef DENSE_VECTOR_H
#define DENSE_VECTOR_H

#include <algorithm>



//! Algevra class collection.
namespace algevra
{


class DenseRefMatrix;
class DenseMatrix;
class SparseMatrix;
class DenseVector;


/** A dense vector reference to another. This vector can be used as a row matrix,
	as a column matrix or as a diagonal matrix.
	Any changes on this vector affects the referenced vector.
	\warning When an operator apply on two DenseRefVector, these vectors MUST have
	the same size. Operators \p != and \p == excluded.
	\sa DenseVector
	\todo Permutation int* array.
	\todo Multiplication with SparseMatrix
*/
struct DenseRefVector
{
	/** C'tor of a vector with \p n elements.
		\param[in] elements Elements of vector. It must remain valid in whole life
		of vector.
		\param n Number of elements in vector.
	*/
	DenseRefVector(double *elements, size_t n) : elements(elements), total(n) {}

	size_t size() const { return total; }	//!< Number of elements.

	//! Return reference of an element in vector.
	double &operator[](size_t p) { return elements[p]; }
	//! Return value from an element in vector.
	double operator[](size_t p) const { return elements[p]; }

	//! Multiply all elements of vector with scalar \p s, and return the result.
	DenseRefVector &operator*=(double s);
	//! Divide all elements of vector with scalar \p s, and return the result.
	DenseRefVector &operator/=(double s) { return *this *= (1 / s); }

	//! Return the multiplication of this vector with scalar \p s.
	DenseVector operator*(double s) const;
	//! Return the division of this vector with scalar \p s.
	DenseVector operator/(double s) const;

	//! Return this vector.
	const DenseRefVector &operator+() const {return *this; }
	//! Return negative of this vector.
	DenseVector operator-() const;

	//! Check if vectors are equal.
	bool operator==(const DenseRefVector &v) const;
	//! Check if vectors are unequal.
	bool operator!=(const DenseRefVector &v) const {return !(*this == v); }

	//! Add vector \p v to this vector, and return the result.
	DenseRefVector &operator+=(const DenseRefVector &v);

	//! Subtrack vector \p v from this vector, and return the result.
	DenseRefVector &operator-=(const DenseRefVector &v);

	//! Return the sum of this vector with vector \p v.
	DenseVector operator+(const DenseRefVector &v) const;
	//! Return the subtraction of vector \p v from this vector.
	DenseVector operator-(const DenseRefVector &v) const;

	//! Return the dot product of this vector with vector \p v.
	double operator*(const DenseRefVector &v) const;
	//! Return the dot product of this vector with vector \p v.
	double operator*(const class SparseVector &v) const;
	//! Return the multiplication of this vector with matrix \p m.
	DenseVector operator*(const DenseRefMatrix &m) const;
	//! Return the multiplication of this vector with matrix \p m.
	DenseVector operator*(const SparseMatrix &m) const;

	//! Return the multiplication of diagonal matrix \p m with this vector.
	DenseVector operator%(const DenseRefVector &m) const;
	//! Multiply diagonal matrix \p m with this vector and return the result.
	DenseRefVector &operator%=(const DenseRefVector &m);
	//! Return the multiplication of diagonal matrix \p m with this vector.
	SparseVector operator%(const SparseVector &m) const;
	//! Return the multiplication of this diagonal matrix with matrix \p m.
	DenseMatrix operator%(const DenseRefMatrix &m) const;
	//! Return the multiplication of this diagonal matrix with matrix \p m.
	SparseMatrix operator%(const SparseMatrix &m) const;

	/** Fill the vector with elements from a buffer.
		\param v Source buffer.
		\param n Copy \p n elements. Default is until the end of vector (-1).
		\param from Copy to this index in target vector. Default is 0.
	*/
	void setRegion(const double *v, size_t n = -1, size_t from = 0);
	/** Fill the vector with elements from another vector.
		\param v Source vector.
		\param n Copy \p n elements. Default is all or until the end of vector,
		what happens first (-1).
		\param from_source Start copying from this index in source vector. Default is 0.
		\param from_target Copy to this index in target vector. Default is 0.
	*/
	void setRegion(const DenseRefVector &v, size_t n = -1, size_t from_source = 0, size_t from_target = 0);
	/** Fill the vector with a value.
		\param value Fill value. Default is 0.
		\param from Fill from index. Default is first.
		\param n Fill \p n elements. Default is all (-1).
	*/
	void setRegion(double value = 0, size_t from = 0, size_t n = -1);

protected:
	double *elements;	//!< Elements.
	size_t total;		//!< Number of elements.
	friend DenseVector;
	friend DenseRefMatrix;
	friend DenseMatrix;
};



/** A dense vector.
	This vector can be used as a row matrix, as a column matrix or as a diagonal
	matrix.
	\warning When an operator apply on two DenseVector, these vectors MUST have
	the same size. Operators \p != and \p == excluded.
	\sa SparseVector, DenseRefVector
*/
struct DenseVector : public DenseRefVector
{
	//! C'tor of a vector with \p n undefined elements.
	//! \param n Number of elements in vector.
	DenseVector(size_t n) : DenseRefVector(0, n) { elements = new double[total]; }

	/** C'tor of a vector with \p n elements.
		\param[in] elements Elements of vector.
		\param n Number of elements in vector.
	*/
	DenseVector(double *elements, size_t n) : DenseRefVector(new double[n], n)
		{ setRegion(elements, n); }

	/** Constructor from a SparseVector.
		\param v Source vector.
		\param n Size of created vector. Default is the size of v (-1).
	*/
	DenseVector(const SparseVector &v, size_t n = -1);

	/** Copy constructor.
		Copy \p n elements of vector \p m, from \p from to new vector.
		\param v Source vector.
		\param n Elements copied from source vector. Default is all (-1).
		\param from Start copy elements from this position in source vector. Default is 0.
	*/
	DenseVector(const DenseRefVector &v, size_t n = -1, size_t from = 0)
		: DenseVector(std::min(v.total - from, n)) { setRegion(v, total, from); }

	//! Move constructor.
	DenseVector(DenseVector &&v) : DenseRefVector(v.elements, v.total) { v.elements = 0; }
	//! Move constructor.
	explicit DenseVector(DenseMatrix &&m);

	~DenseVector() { delete[] elements; }	//!< D'tor.

	//! Resize vector to \p n elements.
	void resize(size_t n);

	//! Copy-assign vector to current.
	DenseVector &operator=(const DenseRefVector &v);
	//! Move-assign vector to current.
	DenseVector &operator=(DenseVector &&v)
		{ delete[] elements; total = v.total; elements = v.elements; v.elements = 0; return *this; }
};


//! Return the multiplication of vector \p v with scalar \p s.
inline DenseVector operator*(double s, const DenseRefVector &v) { return v * s; }

// inline
inline DenseVector DenseRefVector::operator/(double s) const { return *this * (1 / s); }
inline DenseVector DenseRefVector::operator-() const { return *this * -1; }


};	// namespace


#endif // DENSE_VECTOR_H
